package sun.java2d.pipe;

import java.awt.geom.PathIterator;
import sun.awt.geom.PathConsumer2D;
import java.awt.BasicStroke;
import java.awt.geom.AffineTransform;
import java.awt.Shape;
import sun.security.action.GetPropertyAction;
import java.security.AccessController;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.security.PrivilegedAction;

public abstract class RenderingEngine
{
    private static RenderingEngine reImpl;
    
    public RenderingEngine() {
    }
    
    public static synchronized RenderingEngine getInstance() {
        if (RenderingEngine.reImpl != null) {
            return RenderingEngine.reImpl;
        }
        RenderingEngine.reImpl = (RenderingEngine)AccessController.doPrivileged(new PrivilegedAction<RenderingEngine>() {
            @Override
            public RenderingEngine run() {
                String property = System.getProperty("sun.java2d.renderer", "sun.dc.DuctusRenderingEngine");
                if (property.equals("sun.dc.DuctusRenderingEngine")) {
                    try {
                        return (RenderingEngine)Class.forName("sun.dc.DuctusRenderingEngine").newInstance();
                    }
                    catch (ReflectiveOperationException ex) {}
                }
                ServiceLoader<RenderingEngine> loadInstalled = ServiceLoader.loadInstalled(RenderingEngine.class);
                RenderingEngine renderingEngine = null;
                Iterator<Object> iterator = loadInstalled.iterator();
                while (iterator.hasNext() && !(renderingEngine = (RenderingEngine)iterator.next()).getClass().getName().equals(property)) {}
                return renderingEngine;
            }
        });
        if (RenderingEngine.reImpl == null) {
            throw new InternalError("No RenderingEngine module found");
        }
        if ((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.renderer.trace")) != null) {
            RenderingEngine.reImpl = new RenderingEngine.Tracer(RenderingEngine.reImpl);
        }
        return RenderingEngine.reImpl;
    }
    
    public abstract Shape createStrokedShape(Shape p0, float p1, int p2, int p3, float p4, float[] p5, float p6);
    
    public abstract void strokeTo(Shape p0, AffineTransform p1, BasicStroke p2, boolean p3, boolean p4, boolean p5, PathConsumer2D p6);
    
    public abstract AATileGenerator getAATileGenerator(Shape p0, AffineTransform p1, Region p2, BasicStroke p3, boolean p4, boolean p5, int[] p6);
    
    public abstract AATileGenerator getAATileGenerator(double p0, double p1, double p2, double p3, double p4, double p5, double p6, double p7, Region p8, int[] p9);
    
    public abstract float getMinimumAAPenSize();
    
    public static void feedConsumer(PathIterator pathIterator, PathConsumer2D pathConsumer2D) {
        float[] array = new float[6];
        while (!pathIterator.isDone()) {
            switch (pathIterator.currentSegment(array)) {
                case 0: {
                    pathConsumer2D.moveTo(array[0], array[1]);
                    break;
                }
                case 1: {
                    pathConsumer2D.lineTo(array[0], array[1]);
                    break;
                }
                case 2: {
                    pathConsumer2D.quadTo(array[0], array[1], array[2], array[3]);
                    break;
                }
                case 3: {
                    pathConsumer2D.curveTo(array[0], array[1], array[2], array[3], array[4], array[5]);
                    break;
                }
                case 4: {
                    pathConsumer2D.closePath();
                    break;
                }
            }
            pathIterator.next();
        }
    }
    
    static class Tracer extends RenderingEngine
    {
        RenderingEngine target;
        String name;
        
        public Tracer(RenderingEngine target) {
            this.target = target;
            this.name = target.getClass().getName();
        }
        
        @Override
        public Shape createStrokedShape(Shape shape, float f, int i, int j, float f2, float[] obj, float f3) {
            System.out.println(this.name + ".createStrokedShape(" + shape.getClass().getName() + ", width = " + f + ", caps = " + i + ", join = " + j + ", miter = " + f2 + ", dashes = " + obj + ", dashphase = " + f3 + ")");
            return this.target.createStrokedShape(shape, f, i, j, f2, obj, f3);
        }
        
        @Override
        public void strokeTo(Shape shape, AffineTransform obj, BasicStroke obj2, boolean b, boolean b2, boolean b3, PathConsumer2D pathConsumer2D) {
            System.out.println(this.name + ".strokeTo(" + shape.getClass().getName() + ", " + obj + ", " + obj2 + ", " + (b ? "thin" : "wide") + ", " + (b2 ? "normalized" : "pure") + ", " + (b3 ? "AA" : "non-AA") + ", " + pathConsumer2D.getClass().getName() + ")");
            this.target.strokeTo(shape, obj, obj2, b, b2, b3, pathConsumer2D);
        }
        
        @Override
        public float getMinimumAAPenSize() {
            System.out.println(this.name + ".getMinimumAAPenSize()");
            return this.target.getMinimumAAPenSize();
        }
        
        @Override
        public AATileGenerator getAATileGenerator(Shape shape, AffineTransform obj, Region obj2, BasicStroke obj3, boolean b, boolean b2, int[] array) {
            System.out.println(this.name + ".getAATileGenerator(" + shape.getClass().getName() + ", " + obj + ", " + obj2 + ", " + obj3 + ", " + (b ? "thin" : "wide") + ", " + (b2 ? "normalized" : "pure") + ")");
            return this.target.getAATileGenerator(shape, obj, obj2, obj3, b, b2, array);
        }
        
        @Override
        public AATileGenerator getAATileGenerator(double d, double d2, double d3, double d4, double d5, double d6, double d7, double d8, Region obj, int[] array) {
            System.out.println(this.name + ".getAATileGenerator(" + d + ", " + d2 + ", " + d3 + ", " + d4 + ", " + d5 + ", " + d6 + ", " + d7 + ", " + d8 + ", " + obj + ")");
            return this.target.getAATileGenerator(d, d2, d3, d4, d5, d6, d7, d8, obj, array);
        }
    }
}
